% =========================================================================
%   Lei Zhang, Meng Yang, and Xiangchu Feng,
%   "Sparse Representation or Collaborative Representation: Which Helps Face
%    Recognition?" in ICCV 2011.
%
%
% Written by Meng Yang @ COMP HK-PolyU
% July, 2011.
% =========================================================================
                      
close all;
clear all;
clc;

% -------------------------------------------------------------------------
% parameter setting
par.nClass        =   100;                 % the number of classes in the subset of AR database
par.nDim          =   4980;                 % the eigenfaces dimension

%--------------------------------------------------------------------------
%data loading (here we use the AR dataset as an example)
load(['AR_database']);
Tr_DAT   =   [];
trls     =   [];
for ci   =   1:par.nClass
    cdat = Tr_dataMatrix(:,Tr_sampleLabels==ci);
    for i = 1:7
        img = reshape(cdat(:,i),[165 120]);
        img = imresize(img,[83 60]);
        Tr_DAT = [Tr_DAT img(:)];
        trls   = [trls ci];
    end
end

load(['AR_database_Occlusion']);
Tt_DAT_sun   =   [];
ttls_sun     =   [];
for ci   =   1:par.nClass
    cdat1 = Tr_dataMatrix(:,Tr_sampleLabels==ci);
    cdat1 = cdat1(:,1:3);
    cdat2 = Tt_dataMatrix(:,Tt_sampleLabels==ci);
    cdat2 = cdat2(:,1:3);
    cdat  = [cdat1 cdat2];
    for i = 1:6
        img = reshape(cdat(:,i),[165 120]);
        img = imresize(img,[83 60]);
        Tt_DAT_sun = [Tt_DAT_sun img(:)];
        ttls_sun   = [ttls_sun ci];
    end
end

Tt_DAT_sca   =   [];
ttls_sca     =   [];
for ci   =   1:par.nClass
    cdat1 = Tr_dataMatrix(:,Tr_sampleLabels==ci);
    cdat1 = cdat1(:,4:6);
    cdat2 = Tt_dataMatrix(:,Tt_sampleLabels==ci);
    cdat2 = cdat2(:,4:6);
    cdat  = [cdat1 cdat2];
    for i = 1:6
        img = reshape(cdat(:,i),[165 120]);
        img = imresize(img,[83 60]);
        Tt_DAT_sca = [Tt_DAT_sca img(:)];
        ttls_sca   = [ttls_sca ci];
    end
end

tr_dat   =   double(Tr_DAT);
tt_dat   =   double(Tt_DAT_sun); ttls = ttls_sun;
% tt_dat   =   double(Tt_DAT_sca); ttls = ttls_sca;

%--------------------------------------------------------------------------
tr_dat  =  tr_dat./( repmat(sqrt(sum(tr_dat.*tr_dat)), [par.nDim,1]) );
tt_dat  =  tt_dat./( repmat(sqrt(sum(tt_dat.*tt_dat)), [par.nDim,1]) );

KAPPA = [0.001];
for ki = 1:size(KAPPA,2)
    kappa = KAPPA(ki);
%-------------------------------------------------------------------------
%projection matrix computing
Proj_M = inv(tr_dat'*tr_dat+kappa*eye(size(tr_dat,2)))*tr_dat';
%-------------------------------------------------------------------------
%testing
ID = [];
for indTest = 1:size(tt_dat,2)
    tic
    [id]    = CRC_RLS(tr_dat,Proj_M,tt_dat(:,indTest),trls);
    ID      =   [ID id];
    toc
end
cornum      =   sum(ID==ttls);
reco_rate         =   [cornum/length(ttls)]; % recognition rate
fid = fopen([cd '/CRC_AR_disguise2.txt'],'a');
fprintf(fid,'--------------sunglass:700-600----------------------------------------\n');
fprintf(fid,'%s%f%s%f\n',' kappa = ',kappa, 'reco_rate = ',reco_rate);
fclose(fid);
end