% =========================================================================
%   Lei Zhang, Meng Yang, and Xiangchu Feng,
%   "Sparse Representation or Collaborative Representation: Which Helps Face
%    Recognition?" in ICCV 2011.
%
%   Collaborative Representation based Classification for Face Recognition
%   Lei Zhang, Meng Yang, Xiangchu Feng, Yi Ma, David Zhang, ARXIV: http://arxiv.org/abs/1204.2358v2
%
% Written by Meng Yang @ COMP HK-PolyU
% July, 2011 & March, 2014.
% =========================================================================
                      
close all;
clear all;
clc;

% -------------------------------------------------------------------------
% parameter setting
par.nClass        =   100;                 % the number of classes in the subset of AR database
par.nDim          =   300;                 % the eigenfaces dimension
%kappa             =   [0.001];             % kappa*|e|1 + |coef|_2^2;
MaxI              =   70;
%--------------------------------------------------------------------------
%data loading (here we use the AR dataset as an example)
load(['AR_DAT']);
Tr_DAT   =   double(NewTrain_DAT(:,trainlabels<=par.nClass));
trls     =   trainlabels(trainlabels<=par.nClass);
Tt_DAT   =   double(NewTest_DAT(:,testlabels<=par.nClass));
ttls     =   testlabels(testlabels<=par.nClass);
clear NewTest_DAT NewTrain_DAT testlabels trainlabels

%--------------------------------------------------------------------------
%eigenface extracting
[disc_set,disc_value,Mean_Image]  =  Eigenface_f(Tr_DAT,par.nDim);
tr_dat  =  disc_set'*Tr_DAT;
tt_dat  =  disc_set'*Tt_DAT;
tr_dat  =  tr_dat./( repmat(sqrt(sum(tr_dat.*tr_dat)), [par.nDim,1]) );
tt_dat  =  tt_dat./( repmat(sqrt(sum(tt_dat.*tt_dat)), [par.nDim,1]) );

%-------------------------------------------------------------------------
%projection matrix computing
opts.disp = 0;
G         = tr_dat'*tr_dat;
tau       = eigs(double(G),1,'lm',opts);
rho       = 1.25;
tau_input = tau;
mu        = 1/tau;
mubar     = 1e10*mu;
for i = 1:MaxI
    i
    tau = 2/mu;
    Proj(i).M = single(inv(G+tau*eye(size(tr_dat,2)))); % way 1a
%     Proj(i).M = single(inv(G+0.001*eye(size(tr_dat,2)))); % way 1b
%     Proj(i).M = single(inv(G+tau*eye(size(tr_dat,2)))*tr_dat'); % way 2
    mu        = min(rho*mu,mubar);
end

%-------------------------------------------------------------------------
KAPPA = [2];
for ki = 1:size(KAPPA,2)
    kappa = KAPPA(ki);
%testing
ID = [];
for indTest = 1:size(tt_dat,2)
    tic
    [id]    = RCRC_RLS(tr_dat,Proj,tt_dat(:,indTest),trls,kappa,tau_input,MaxI-1);
    ID      =   [ID id];
    toc
end
cornum      =   sum(ID==ttls);
reco_rate         =   [cornum/length(ttls)]; % recognition rate
fid = fopen([cd '/R_CRC_AR_.txt'],'a');
fprintf(fid,'--------------700-700--1b--------------------------------------\n');
fprintf(fid,'%s%f%s%f\n',' kappa = ',kappa, 'reco_rate = ',reco_rate);
fclose(fid);
end